"use strict";
/**
 *
 * Wechaty: Wechat for Bot. and for human who talk to bot/robot
 *
 * Class StateSwitch
 * Licenst: Apache-2.0
 * https://github.com/zixia/state-switcher
 *
 * Helper Class for Manage State Change
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A - State A
 * B - State B
 */
class StateSwitch {
    constructor(_client, initState, logInstance) {
        this._client = _client;
        if (logInstance) {
            this.setLog(logInstance); // init log, should be a Brolog or Npmlog instance
        }
        else {
            this.setLog(null);
        }
        this.log.silly('StateSwitch', 'constructor(%s, %s)', _client, initState);
        this._target = initState;
        this._current = initState;
        this._stable = true;
    }
    setLog(logInstance) {
        if (logInstance) {
            this.log = logInstance;
        }
        else {
            this.log = {
                silly(...args) { },
                verbose(...args) { },
                warn(...args) { },
                error(...args) { },
            };
        }
    }
    /**
     * set/get target state
     */
    target(newState) {
        if (newState) {
            this.log.verbose('StateSwitch', '%s:target(%s) <- (%s)', this._client, newState, this._target);
            this._target = newState;
        }
        else {
            this.log.silly('StateSwitch', '%s:target() - %s', this._client, this._target);
        }
        return this._target;
    }
    /**
     * set/get current state
     * @param stable boolean  true for stable, false for inprocess
     */
    current(newState, stable = true) {
        if (newState) {
            this.log.verbose('StateSwitch', '%s:current(%s,%s) <- (%s,%s)', this._client, newState, stable, this._current, this._stable);
            /**
             * strict check current is equal to target
             */
            if (this._target !== newState) {
                this.log.warn('StateSwitch', '%s:current(%s,%s) current is different with target. call state.target(%s) first.', this._client, newState, stable, newState);
                const e = new Error('current not match target');
                this.log.verbose('StateSwitch', e.stack);
                throw e;
            }
            /**
             * warn for inprocess current state change twice, mostly like a logic bug outside
             */
            if (this._current === newState && this._stable === stable
                && stable === false) {
                this.log.warn('StateSwitch', '%s:current(%s,%s) called but there are already in the same state', this._client, newState, stable);
                const e = new Error('current unchange');
                this.log.verbose('StateSwitch', e.stack);
            }
            this._current = newState;
            this._stable = stable;
        }
        else {
            this.log.silly('StateSwitch', '%s:current() - %s', this._client, this._current);
        }
        return this._current;
    }
    /**
     * does the current state be stable(not inprocess)?
     */
    stable() {
        this.log.silly('StateSwitch', '%s:stable() is %s', this._client, this._stable);
        return this._stable;
    }
    /**
     * does the current state be inprocess(not stable)?
     */
    inprocess() {
        this.log.silly('StateSwitch', '%s:inprocess() %s', this._client, !this._stable);
        return !this._stable;
    }
    /**
     * get the client name
     */
    client() {
        return this._client;
    }
}
exports.StateSwitch = StateSwitch;
exports.default = StateSwitch;
//# sourceMappingURL=state-switch.js.map